# ----------------------------
# constants
# ----------------------------

FILENAME = 'movies.txt'

# ----------------------------
# list utilities from Task 2.2
# ----------------------------

def list_union(list_1, list_2):
    result = []
    for item in list_1:
        result.append(item)
    for item in list_2:
        if item not in list_1:
            result.append(item)
    return result

def list_intersect(list_1, list_2):
    result = []
    for item in list_1:
        if item in list_2:
            result.append(item)
    return result

# ----------------------------------
# Task 1.2 -- build_actor_database()
# ----------------------------------

def build_actor_database(filename):
    actor_db = {}

    source   = open(filename, 'r')
    for line in source:
        data_str = line.strip().split(',')
        actor  = data_str[0].strip().title()
        movies = data_str[1:]
        actor_db[actor] = []
        for movie in movies:
            movie = movie.strip().title()
            actor_db[actor].append(movie)
    source.close()
    
    return actor_db

# ----------------------------------
# Task 1.3 -- build_movie_database()
# ----------------------------------

def build_movie_database(actor_db):
    movie_db = {}

    for actor in actor_db:
        movies = actor_db[actor]
        for movie in movies:
            if movie not in movie_db:
                movie_db[movie] = [actor]
            else:
                movie_db[movie].append(actor)

    return movie_db

# ----------------------------
# Task 2.1 -- compare_movies()
# ----------------------------

def compare_movies(movie_1, movie_2):
    if movie_1 not in movie_db:
        print('Sorry, but', movie_1, 'is not in the database.')
        return
    if movie_2 not in movie_db:
        print('Sorry, but', movie_2, 'is not in the database.')
        return

    actors_1 = movie_db[movie_1]
    actors_2 = movie_db[movie_2]

    print('\nAll actors in', movie_1, 'and', movie_2, ':\n')
    for actor in list_union(actors_1, actors_2):
        print('   ', actor)

    print('\nAll actors in BOTH', movie_1, 'and', movie_2, ':\n')
    for actor in list_intersect(actors_1, actors_2):
        print('   ', actor)

# -----------------------------
# Task 3.1 -- acting_partners()
# -----------------------------

def acting_partners(actor):
    if actor not in actor_db:
        print('Sorry, but', actor, 'is not in the database.')
        return

    partners = []
    for movie in actor_db[actor]:
        actors = movie_db[movie]
        partners = list_union(partners, actors)
    partners.remove(actor)
    partners.sort()

    print('\nAll actors in a movie with', actor, ':\n')
    for partner in partners:
        print('   ', partner)

# ----------------------------
# Task 3.2 -- compare_actors()
# ----------------------------

def compare_actors(actor_1, actor_2):
    if actor_1 not in actor_db:
        print('Sorry, but', actor_1, 'is not in the database.')
        return
    if actor_2 not in actor_db:
        print('Sorry, but', actor_2, 'is not in the database.')
        return

    movies_1 = actor_db[actor_1]
    movies_2 = actor_db[actor_2]

    print('\nAll movies in which BOTH', actor_1, 'and', actor_2, 'appear:\n')
    for movie in list_intersect(movies_1, movies_2):
        print('   ', movie)

# ----------------------------------
# DEMO CODE
# ----------------------------------

actor_db = build_actor_database(FILENAME)
movie_db = build_movie_database(actor_db)

print('\nTask 1.2 -- ACTOR DATABASE\n')
for key, value in actor_db.items():
    print(key, value)

print('\nTask 1.3 -- MOVIE DATABASE\n')
for key, value in movie_db.items():
    print(key, value)
print()

print('\nTask 2.1 -- COMPARE MOVIES\n')
compare_movies('A Few Good Men', 'Sleepers')

print('\nTask 3.1 -- ACTING PARTNERS\n')
acting_partners('Tom Cruise')

print('\nTask 3.2 -- COMPARE ACTORS\n')
compare_actors('Tom Cruise', 'Kevin Bacon')
compare_actors('Morgan Freeman', 'Brad Pitt')
compare_actors('Tom Hanks', 'Meg Ryan')
